<?php 
/**
 * Dynamic CSS Filter
 *
 * An open source dynamic CSS filter.
 *
 * @author		Kepler Gelotte
 * @copyright	Copyright (c) 2008, Neighbor Webmaster, Inc.
 * @license		http://www.coolphptools.com/dynamic_css/license
 * @link		http://www.coolphptools.com
 * @version		Version 2.9
 */
$debug = false;
$cache = true;
$allow_eol_comments = true;
$compress = true;
$compress_comments = true;
$handle_pngs = true;
// NOTE: I have had no luck using the alpha filter!
$use_alpha_filter = false; // Use the alpha filter for IE5.5 & IE6?
$filter_sizing_method = 'scale'; // valid values are 'crop', 'image', 'scale'
$convert_to_png8 = false; // If set to true creates an 8-bit PNG, otherwise creates a GIF
$matte_color = 'fff'; // matte color or alpha blending on transparent images

// see if any settings were overridden
$system_boolean_parameters = array('debug', 'cache', 'allow_eol_comments', 'compress', 'compress_comments', 'handle_pngs', 'use_alpha_filter', 'convert_to_png8');
$system_string_parameters = array('filter_sizing_method');
if (isset($_SERVER['QUERY_STRING']))
{
	$arr = explode('&', ltrim($_SERVER['QUERY_STRING'], '?'));
	foreach ($arr as $arg)
	{
		if (($pos = strpos($arg, '=')) > 0)
		{
			$key = substr($arg, 0, $pos);
			$value = substr($arg, $pos+1);
			if (in_array($key, $system_boolean_parameters))
			{
				if (strcasecmp($value, 'true') === 0 || $value == 1)
				{
					$$key = true;
				}
				else
				{
					$$key = false;
				}
			}
			elseif (in_array($key, $system_string_parameters))
			{
				$$key = $value;
			}
		}
	}
}

function filter( $input )
{
	global $debug, $cache, $allow_eol_comments, $compress, $compress_comments, $handle_pngs, $matte_color; // , $start, $expires, $output;

	header("Content-type: text/css;");
	header("Cache-Control: must-revalidate;");

	if ( $cache )
	{
		// see if we need a browser specific cache file
		$browser = '';
		if ( $handle_pngs )
		{
			$msie_no_filter = '/msie\s([1-4]|5\.[0-4]).*(win)/i';
			$msie = '/msie\s(5\.[5-9]|6\.).*(win)/i';
			$msie_ok = '/msie\s[7-9].*(win)/i';
			$msie_no_filter = '/msie\s([0-4]\.|5\.[0-4]).*(win)/i';
			$opera = '/opera/i';

			if ( isset($_SERVER['HTTP_USER_AGENT']) )
			{
				if ( preg_match($msie, $_SERVER['HTTP_USER_AGENT'])
					&& ! preg_match($msie_ok, $_SERVER['HTTP_USER_AGENT'])
					&& ! preg_match($opera, $_SERVER['HTTP_USER_AGENT']) )
				{
					$browser = 'ie6_';
				}
				elseif ( preg_match($msie_no_filter, $_SERVER['HTTP_USER_AGENT'])
					&& ! preg_match($msie_ok, $_SERVER['HTTP_USER_AGENT'])
					&& ! preg_match($opera, $_SERVER['HTTP_USER_AGENT']) )
				{
					$browser = 'ie5_';
				}
			}
		}

		$cache_dir = getcwd()."/cache/";
		$cache_file = $cache_dir.$browser.str_replace('?', '_', str_replace('&', '_', basename($_SERVER['REQUEST_URI'])));
		$dependency_file = $cache_dir.$browser."dependency_".str_replace('?', '_', str_replace('&', '_', basename($_SERVER['REQUEST_URI'])));
		$css_file = getcwd()."/".$browser.preg_replace('/\?.*/', '', basename($_SERVER['REQUEST_URI']));
		if ( file_exists( $cache_file ) )
		{
			$cache_stat = stat( $cache_file );
			$use_cache_file = true;

			// first see if the main CSS file is newer than cache
			$css_stat = stat( $css_file );
			if ( $cache_stat['mtime'] < $css_stat['mtime'] )
			{
				$use_cache_file = false;
			}

			if ( $use_cache_file && file_exists( $dependency_file ) )
			{
				$fp = fopen( $dependency_file, 'r' );
				$contents = fread($fp, filesize($dependency_file));
				fclose($fp);
				if ( $dependent_files = @unserialize( $contents ) )
				{
					foreach ( $dependent_files as $file => $stat )
					{
						$css_stat = stat( getcwd()."/".$file );
						if ( ! $css_stat || $css_stat['mtime'] > $stat['mtime'] )
						{
							$use_cache_file = false;
							break;
						}
					}
				}
			}

			// is the cache file still Ok?
			if ( $use_cache_file )
			{
				$output = file_get_contents( $cache_file );
				return "/* from cache */\n".$output;
			}
		}
		$dependent_files = array();
	}

	$output = "";

	$expires = 31536000; // default expires in 1 year
	$start = microtime(true);
	// session_start();
	$level = 0;
	$lines[$level] = explode( "\n", $input );
	if ($debug)
	{
		$output = "/* DEBUG filter start ".strftime("%H:%M:%S")." */\n";
	}
	$vars = array();

	// treat any passed parameters as variables
	if (isset($_SERVER['QUERY_STRING']))
	{
		$arr = explode('&', ltrim($_SERVER['QUERY_STRING'], '?'));
		foreach ($arr as $arg)
		{
			if (($pos = strpos($arg, '=')) > 0)
			{
				$key = substr($arg, 0, $pos);
				$value = substr($arg, $pos+1);
				$vars[$key] = $value;
			}
		}
	}

	$line_number = 0;
	$if_level = 0;
	$ifs = array();
	$prev_line = '';
	while ( $level >= 0 )
	{
		while ( list( $index, $line ) = each( $lines[$level] ) )
		{
			$line_number++;

			if ($allow_eol_comments)
			{
				if ($compress_comments)
				{
					// remove end-of-line comments (like this one)
					$line = preg_replace( '#//.*#', '', $line );
				}
				else
				{
					// convert end-of-line comments to regular comments
					$line = preg_replace( '#//\s*([^\r\n]*)\s*#', "/* \${1} */", $line );
				}
			}

			// see if the newline was escaped - if so, concatenate the line
/*
if (strrpos($line, "\\"))
{
	$output .= "\n/* DEBUG strrpos = ".strrpos($line, '\\').' strlen = '.(strlen($line) - 2)." line=".$line."*"."/\n";
}
*/
			$line = rtrim($line, "\r\n");
			if ($line{(strlen($line) - 1)} == "\\")
			{
				$prev_line .= substr($line, 0, -2);
				continue;
			}
			$line = $prev_line . $line;
			$prev_line = '';

			$line = substitute_delimited_vars( $line, $vars );
			$clauses = preg_split( '/([\{\};])/', str_replace('\\;', '<--|-->', $line), -1, PREG_SPLIT_DELIM_CAPTURE );
			$suppress_delimiter = false;
			foreach( $clauses as $clause )
			{
			$clause = str_replace('<--|-->', ';', $clause);
if ($debug) {$output .= "/* DEBUG line: {$line_number} Clause=<".$clause."> - should execute? ".((should_execute( $ifs, $if_level ))?"YES":"NO")." */\n";}
			if (preg_match( '#^\s*expires\s*(.*)#i', $clause, $matches) === 1)
			{
				$suppress_delimiter = true;
				if ( should_execute( $ifs, $if_level ) )
				{
					$expires = rtrim($matches[1], "\r\n\t ;");
					$expires = substitute_vars( $expires, $vars );
					$expires = preg_replace( '#immediate|yesterday#i', '-86400', $expires );
					$expires = preg_replace( '#now#i', '0', $expires );
					$expires = preg_replace( '#minute[s]?#i', '*60', $expires );
					$expires = preg_replace( '#hour[s]?#i', '*3600', $expires );
					$expires = preg_replace( '#day[s]?#i', '*86400', $expires );
					$expires = preg_replace( '#week[s]?#i', '*604800', $expires );
					$expires = preg_replace( '#month[s]?#i', '*2592000', $expires ); // approx.
					$expires = preg_replace( '#year[s]?#i', '*31536000', $expires );
					$expires = eval( 'return (int)'.$expires.';' );

					if ($debug)
					{
						$output .= "/* DEBUG expires = ".$expires." line: ".$line_number." */\n";
					}
				}
			}
			else if (preg_match( '#^\s*header\s*(.*)#i', $clause, $matches) === 1)
			{
				$suppress_delimiter = true;
				if ( should_execute( $ifs, $if_level ) )
				{
					$header = rtrim($matches[1], "\r\n\t ;");
					$header = substitute_vars( $header, $vars );
					header($header);

					if ($debug)
					{
						$output .= "/* DEBUG header = ".$header." line: ".$line_number." */\n";
					}
				}
			}
			else if (preg_match( '#^\s*set\s+\$?([_a-z][a-z0-9_-]*)\s*=?\s*(.*)\s*#i', $clause, $matches) === 1)
			{
				$suppress_delimiter = true;
				if ( should_execute( $ifs, $if_level ) )
				{
					$key = $matches[1];
					$value = rtrim($matches[2], "\r\n\t ;");
					if ($debug)
					{
						$output .= "/* DEBUG set ".$key." = ".$value." */\n";
					}
					$vars[$key] = $value;
				}
			}
			else if (preg_match( '#^\s*eval\s+\$?([_a-z][a-z0-9_-]*)\s*=?\s*(.*)#i', $clause, $matches) === 1)
			{
				$suppress_delimiter = true;
				if ( should_execute( $ifs, $if_level ) )
				{
					$key = $matches[1];
					$value = rtrim($matches[2], "\r\n\t ;");
					$expr = eval ( 'return '.substitute_vars($value, $vars).';' );
					if ($debug)
					{
						$output .= "/* DEBUG eval ".$key." = ".$value." (".$expr.") */\n";
					}
					$vars[$key] = $expr;
				}
			}
			else if (preg_match( '#^\s*eval\s+(.*)#i', $clause, $matches) === 1)
			{
				$suppress_delimiter = true;
				if ( should_execute( $ifs, $if_level ) )
				{
					$value = rtrim($matches[1], "\r\n\t ;");
					$expr = eval ( 'return '.substitute_vars($value, $vars).';' );
					if ($debug)
					{
						$output .= "/* DEBUG eval ".$value." (".$expr.") */\n";
					}
				}
			}
			else if (preg_match( '#^\s*matte-color:\s+\#([0-9abcdef]{6}|[0-9abcdef]{3})\s*#i', $clause, $matches) === 1)
			{
				$suppress_delimiter = true;
				if ( should_execute( $ifs, $if_level ) )
				{
					$matte_color = $matches[1];
					if ($debug)
					{
						$output .= "/* DEBUG matte-color: ".$matte_color." */\n";
					}
				}
			}
			else if (preg_match( '#^\s*if\s+(.*)\s*#i', $clause, $matches) === 1)
			{
				$suppress_delimiter = true;
				if ( should_execute( $ifs, $if_level ) )
				{
					$value = rtrim($matches[1], "\r\n\t ;");
					$expr = substitute_vars( $value, $vars );
					$ifs[$if_level] = eval( "return (boolean)".$expr.";");
				}
				else
				{
					$ifs[$if_level] = false; // save unneccessary evaluation
				}
				$if_level++;
			}
			else if (preg_match( '#^\s*elif\s+(.*)\s*#i', $clause, $matches) === 1
				|| preg_match( '#^\s*elseif\s+(.*)\s*#i', $clause, $matches) === 1)
			{
				$suppress_delimiter = true;
				if ( ! $ifs[($if_level - 1)] )
				{
					$value = rtrim($matches[1], "\r\n\t ;");
					$expr = substitute_vars( $value, $vars );
					$ifs[($if_level - 1)] = eval( "return (boolean)".$expr.";");
				}
				else
				{
					$ifs[($if_level - 1)] = false; // save unneccessary evaluation
				}
			}
			else if (preg_match( '#^\s*else\s*#i', $clause, $matches) === 1)
			{
				$suppress_delimiter = true;
				$ifs[($if_level - 1)] = ! $ifs[($if_level - 1)];
			}
			else if (preg_match( '#^\s*endif\s*#i', $clause, $matches) === 1)
			{
				$suppress_delimiter = true;
				$if_level--;
				if ($if_level < 0)
				{
					$output .= "/* ERROR unmatched endif at line ".$line_number." */";
				}
			}
			else if (preg_match( '#^\s*@include\s*url\(\s*["\']?([^"\'\s]+)["\']?\s*\)\s*#i', $clause, $matches) === 1
				|| preg_match( '#^\s*@include\s*["\']?([^"\'\s]+)["\']?\s*#i', $clause, $matches) === 1)
			{
				$suppress_delimiter = true;
				$value = rtrim($matches[1], "\r\n\t ;");
				$fn = substitute_vars( $value, $vars );
				$level++;
				if ( ($fp = fopen( $fn, 'r' )) !== false )
				{
					if ($debug)
					{
						$output .= "/* DEBUG including style file ".$fn." */";
					}
					$contents = fread($fp, filesize($fn));
					fclose($fp);

					$lines[$level] = explode( "\n", $contents );

					if ( $cache )
					{
						$dependent_files[$fn] = stat( $fn );
					}
				}
			}
			else
			{
				if ( should_execute( $ifs, $if_level ) )
				{
					$clause = substitute_vars( $clause, $vars );

					if ( $handle_pngs && stristr($clause, '.png') )
					{
						$clause = handle_pngs( $clause );
					}
					if ($clause != ';' || ! $suppress_delimiter)
					{
						$output .= trim($clause) . "\n";
						$suppress_delimiter = false;
					}
				}
			}
			}
		}
		$level--;
	}

	$expDate = gmdate("D, d M Y H:i:s", time() + $expires) . " GMT";
	header("Expires: ".$expDate);
	header("Last-Modified: ".$expDate);
	if ($if_level > 0)
	{
		$output .= "/* ERROR ".$if_level." unterminated if statements */\n";
	}
	if ($debug)
	{
		$output .= "/* DEBUG processing time = ".(microtime(true) - $start)." seconds. expires = ".$ExpStr." */\n";
		$output .= "/* DEBUG filter end ".strftime("%H:%M:%S")." */\n";
	}

	if ($compress_comments)
	{
		// remove standard comments
		$output = preg_replace( '#/\*(?!\ DEBUG|\ INFO|\ WARNING|\ ERROR)[^*]*\*+(?:[^/*][^*]*\*+)*/#', '', $output );
		// $output = preg_replace( '#/\*[^*]*\*+([^/*][^*]*\*+)*/#', '', $output );
	}

	if ($compress)
	{
		// remove blank lines
		$output = preg_replace( '#^\s*\n+#', "", $output );
		$output = preg_replace( '#\n+\s*\n+#', "\n", $output );
		// remove extra whitespace
		$output = preg_replace( '#\n?\s*([\{\}:;,])\s*#', "$1", $output );
		// remove extra semicolons
		$output = preg_replace( '#;;*#', ";", $output );
		// remove extra semicolon before close brace
		$output = preg_replace( '#;?(\}+)#', "$1\n", $output );
	}

	if ( $cache )
	{
		if ( ! file_exists( $cache_dir ) )
		{
			mkdir( $cache_dir );
		}
		$rc = file_put_contents( $cache_file, $output );
		// $output .= "/* file put returned ".$rc." */";

		// save the depencies if there are any
		if ( count( $dependent_files ) > 0 )
		{
			$fp = fopen( $dependency_file, 'w' );
			$contents = @serialize( $dependent_files );
			fwrite($fp, $contents);
			fclose($fp);
		}
	}

	return $output;
}

function should_execute( $ifs, $level )
{
	for ($i = 0; $i < $level; $i++)
	{
		if ( ! $ifs[$i] )
		{
			return false;
		}
	}

	return true;
}

function substitute_delimited_vars( $line, $vars = array() )
{
	foreach ( $vars as $key => $value )
	{
		$line = preg_replace( '#\$\{'.$key.'\}#i', str_replace(';', '\\;', $value), $line );
	}

	return $line;
}

function substitute_vars( $line, $vars = array() )
{
	foreach ( $vars as $key => $value )
	{
		$line = preg_replace( '#\$'.$key.'#i', $value, $line );
	}

	return $line;
}

function handle_pngs( $line )
{
	global $debug, $use_alpha_filter, $filter_sizing_method, $convert_to_png8, $matte_color;

// $_SERVER['HTTP_USER_AGENT'] = 'msie 5.6 winNT';

	$msie = '/msie\s(5\.[5-9]|6\.).*(win)/i';
	$msie_ok = '/msie\s[7-9].*(win)/i';
	$msie_no_filter = '/msie\s([0-4]\.|5\.[0-4]).*(win)/i';
	$opera = '/opera/i';

	if ( isset($_SERVER['HTTP_USER_AGENT']) )
	{
		if ( $use_alpha_filter
			&& preg_match($msie, $_SERVER['HTTP_USER_AGENT'])
			&& ! preg_match($msie_ok, $_SERVER['HTTP_USER_AGENT'])
			&& ! preg_match($opera, $_SERVER['HTTP_USER_AGENT'])
			)
		{
			preg_match('/(.*):(.*)url\s*\(\s*[\'"]?([^\'"\);]*)[\'"]?\s*\)(.*)/', $line, $matches);
			if (count($matches) == 5)
			{
				$line = '';
				if ( strcasecmp( trim($matches[1]), 'background-image' ) !== 0 )
				{
					$line .= $matches[1].':'.$matches[2].' '.$matches[4].';';
				}
				$line .= "zoom:100%;";
				$line .= "display:inline-block;";
				return $line.'filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src="'.$matches[3].'",sizingMethod="'.$filter_sizing_method.'",enabled="true");';
			}
			else
			{
				return $line;
			}
		}
		else
		{
			// browser too old to use filters - use gif/png8 image instead
			if ( ! preg_match($msie_ok, $_SERVER['HTTP_USER_AGENT'])
				&& (( ! $use_alpha_filter
						&& preg_match($msie, $_SERVER['HTTP_USER_AGENT'])
					)
					|| preg_match($msie_no_filter, $_SERVER['HTTP_USER_AGENT'])
					)
				)
			{
				$img = preg_replace('/.*url\s*\(\s*[\'"]?([^\'"]*)[\'"]?\s*\).*/', '$1', $line);

				// try to figure out the background color
				$background_color = 'FFFFFF';
				preg_match('/.*:.*#([0-9a-fA-F]{3,6}).*/', $line, $matches);
				if (count($matches) == 2)
				{
					$background_color = $matches[1];
				}

				// Convert to GIF or PNG8?
				if ($convert_to_png8)
				{
					$new_img = preg_replace("/\.png/i", ".png8", $img);
				}
				else
				{
					$new_img = preg_replace("/\.png/i", ".gif", $img);
				}

				if ( ! file_exists( $new_img ) )
				{
					// gif file doesn't exist, let's try to create it
					// Load Image and get it's size
					$size = getimagesize($img);

					// KG - removed because causes process to hang
					// $required_memory = round($size[0] * $size[1] * 2 * $size['bits']);
					// $new_limit = memory_get_usage() + $required_memory;
					// ini_set("memory_limit", $new_limit);
					
					$image = imagecreatefrompng($img);
					$image_result = imagecreatetruecolor($size[0],$size[1]);

					if ( strlen( $matte_color ) == 3 )
					{
						$red = hexdec( $matte_color[0].$matte_color[0] );
						$green = hexdec( $matte_color[1].$matte_color[1] );
						$blue = hexdec( $matte_color[2].$matte_color[2] );
					}
					else
					{
						$red = hexdec( $matte_color[0].$matte_color[1] );
						$green = hexdec( $matte_color[2].$matte_color[3] );
						$blue = hexdec( $matte_color[4].$matte_color[5] );
					}
					$line .= '/* DEBUG: red='.$red.' green='.$green.' blue='.$blue.' */';
					$bga = imagecolorallocate($image_result, $red, $green, $blue, 127); 
					convert_alpha( $image, $red, $green, $blue );

					if ($convert_to_png8)
					{
						imagefill($image_result, 0, 0, $bga); 
						imagetruecolortopalette($image_result, false, 255);
					}

					imagecopyresampled($image_result,$image,0,0,0,0,$size[0],$size[1],$size[0],$size[1]);
					imagecolortransparent($image_result, $bga);
					// imagealphablending($image_result, true);
					// imagesavealpha($image_result, true);

					if ($convert_to_png8)
					{
						imagepng($image_result,$new_img);    
					}
					else
					{
						imagegif($image_result,$new_img);    
					}
					imagedestroy($image);
					imagedestroy($image_result);
					// ini_restore ("memory_limit");
				}
				$line = str_replace( $img, $new_img, $line );
			}
		}
	}

	return $line;
}

function convert_alpha( &$image, $bg_red=255, $bg_green=255, $bg_blue=255 )
{
	$height = imagesy( $image );
	$width = imagesx( $image );
	for($x = 0; $x < $width; $x++)
	{
		for($y = 0; $y < $height; $y++)
		{
			$color = imagecolorat( $image, $x, $y );
			$alpha = ( $color >>24 ) & 0x7F;
			$red   = ( $color >> 16 ) & 0xFF;
			$green = ( $color >> 8 ) & 0xFF;
			$blue  = $color & 0xFF;
			if ( $alpha > 0 && $alpha < 127 )
			{
				$alpha_float = (float)$alpha;
				$new_red = (float)$red * ( (127.0 - $alpha_float) / 127.0 ) +
					(float)$bg_red * ( 1.0 - ( (127.0 - $alpha_float) / 127.0 ) );
				$new_green = (float)$green * ( (127.0 - $alpha_float) / 127.0 ) +
					(float)$bg_green * ( 1.0 - ( (127.0 - $alpha_float) / 127.0 ) );
				$new_blue = (float)$blue * ( (127.0 - $alpha_float) / 127.0 ) +
					(float)$bg_blue * ( 1.0 - ( (127.0 - $alpha_float) / 127.0 ) );
				if ( ( $new_color = imagecolorallocate( $image, (int)$new_red, (int)$new_green, (int)$new_blue ) ) !== false )
				{
					imagesetpixel( $image ,$x, $y, $new_color );
				}
			}
		}
	}
}


ob_start ("filter");
?>
